"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ScopedContext = exports.DependencyTracker = void 0;
var ServiceCollection_1 = require("../ServiceCollection");
var Errors_1 = require("../Errors");
var DependencyTracker = /** @class */ (function () {
    function DependencyTracker() {
        Object.defineProperty(this, "singletons", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: []
        });
        Object.defineProperty(this, "lookup", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: {}
        });
        Object.defineProperty(this, "ordered", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: []
        });
    }
    Object.defineProperty(DependencyTracker.prototype, "depth", {
        get: function () {
            return this.ordered.length;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(DependencyTracker.prototype, "singleton", {
        get: function () {
            if (this.singletons.length === 0)
                return undefined;
            return this.singletons[this.singletons.length - 1];
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(DependencyTracker.prototype, "last", {
        get: function () {
            if (this.ordered.length === 0)
                return undefined;
            return this.ordered[this.ordered.length - 1];
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(DependencyTracker.prototype, "enter", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (lifetime) {
            if (lifetime.name in this.lookup)
                throw new Errors_1.CircularDependencyError(lifetime.name, this.ordered.map(function (e) { return e.name; }));
            this.ordered.push(lifetime);
            if (lifetime.isSingleton)
                this.singletons.push(lifetime);
            this.lookup[lifetime.name] = lifetime;
        }
    });
    Object.defineProperty(DependencyTracker.prototype, "leave", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (lifetime) {
            var last = this.ordered.pop();
            if (last === null || last === void 0 ? void 0 : last.isSingleton)
                this.singletons.pop();
            delete this.lookup[lifetime.name];
        }
    });
    Object.defineProperty(DependencyTracker.prototype, "traverseOnto", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (other) {
            this.ordered.forEach(function (e) { return other.enter(e); });
        }
    });
    Object.defineProperty(DependencyTracker.prototype, "clone", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function () {
            var clone = new DependencyTracker();
            this.ordered.forEach(function (e) { return clone.enter(e); });
            return clone;
        }
    });
    return DependencyTracker;
}());
exports.DependencyTracker = DependencyTracker;
var ScopedContext = /** @class */ (function () {
    function ScopedContext(parent) {
        Object.defineProperty(this, "dependencyTracker", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "lifetimes", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "proxy", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "scope", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "parent", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.parent = parent;
        this.dependencyTracker = new DependencyTracker();
        if (parent instanceof ScopedContext) {
            this.scope = parent.scope;
            parent.dependencyTracker.traverseOnto(this.dependencyTracker);
        }
        else {
            this.scope = {};
        }
        var self = this;
        this.lifetimes = parent.lifetimes;
        this.proxy = new Proxy({}, {
            get: function (target, prop) { return self.provide(prop); }
        });
    }
    Object.defineProperty(ScopedContext.prototype, "provide", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (selector) {
            var key = ServiceCollection_1.ServiceCollection.extractSelector(selector);
            var lifetime = this.lifetimes[key];
            if (!lifetime)
                throw new Errors_1.ExistenceDependencyError(key);
            this.dependencyTracker.enter(lifetime);
            var result = lifetime.provide(this);
            this.dependencyTracker.leave(lifetime);
            return result;
        }
    });
    Object.defineProperty(ScopedContext.prototype, "clone", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function () {
            return new ScopedContext(this);
        }
    });
    return ScopedContext;
}());
exports.ScopedContext = ScopedContext;
